import configparser
import logging
from atgateway.app.sensing_mgr.const import Const


class ModelConfigBase:
    # DEFAULT section
    CLOUD_CONFIG = "cloud_config"
    SEND_CLOUD = "send_cloud"
    CACHE = "cache"
    SEND_INTERVAL = "send_interval"
    DATA_SEND_ONESHOT = "data_send_oneshot"
    WAIT_CONTAINER_STOP = "wait_container_stop"

    # LOG section
    LOG = "LOG"
    FILE = "file"
    STREAM = "stream"

    # ADC section
    ADC = "ADC"
    POLLING_INTERVAL = "polling_interval"


    MAX_INTERVAL = 3600

    def __init__(self):
        self._configs = {}
        self._logging = logging.getLogger(Const.LOGGER_NAME)

    def load(self, config_file_path):
        config = configparser.ConfigParser()
        config.read(config_file_path)

        if not self.parse_default_config(config):
            self._logging.error("config file not found [DEFAULT] section.")
            return False

        self.parse_log_config(config)
        try:
            self.parse_adc_config(config)
        except Exception as e:
            self._logging.error("[ADC] section is invalid.")
            self._logging.error(e.args)
        return True

    def check_max_interval(self, val):
        try:
            if val == "":
                return 1

            if isinstance(val, str):
                val = int(val)
            if val > self.MAX_INTERVAL:
                val = self.MAX_INTERVAL
                msg = print(
                    "max interval set {}[s].".format(self.MAX_INTERVAL))
                self._logging.info(msg)
            return val
        except Exception as e:
            self._logging.error("max interval check error.")
            self._logging.error(e.args)
            return 1

    def parse_default_config(self, config):
        default_dict = {}
        section = "DEFAULT"
        try:
            cloud_config = config[section].getboolean(
                ModelConfigBase.CLOUD_CONFIG)
            default_dict[ModelConfigBase.CLOUD_CONFIG] = cloud_config

            send_cloud = config[section].getboolean(
                ModelConfigBase.SEND_CLOUD)
            default_dict[ModelConfigBase.SEND_CLOUD] = send_cloud

            cache = config[section].getboolean(
                ModelConfigBase.CACHE)
            default_dict[ModelConfigBase.CACHE] = cache

            send_interval = self.check_max_interval(
                config[section].getint(ModelConfigBase.SEND_INTERVAL, 10))
            if send_interval < 1:
                send_interval = 1
            elif send_interval > 10:
                send_interval = 10
            default_dict[ModelConfigBase.SEND_INTERVAL] = send_interval

            oneshot = config[section].getboolean(
                ModelConfigBase.DATA_SEND_ONESHOT, False)
            default_dict[ModelConfigBase.DATA_SEND_ONESHOT] = oneshot

            stop = config[section].getint(
                ModelConfigBase.WAIT_CONTAINER_STOP, 0)
            if stop < 0:
                stop = 0
            elif stop > 60:
                stop = 60
            default_dict[ModelConfigBase.WAIT_CONTAINER_STOP] = stop

            self._configs[section] = default_dict
            return True
        except Exception as e:
            self._logging.error(
                "[DEFAULT] section need [cloud_config], [send_cloud], \
                [cache], [send_interval], [data_send_oneshot],\
                [wait_container_stop]")
            self._logging.error(e.args)
        return False

    def parse_log_config(self, config):
        log_dict = {}
        section = ModelConfigBase.LOG
        file = config[section].getboolean(ModelConfigBase.FILE, True)
        log_dict[ModelConfigBase.FILE] = file
        stream = config[section].getboolean(ModelConfigBase.STREAM, False)
        log_dict[ModelConfigBase.STREAM] = stream
        self._configs[section] = log_dict

    # ADC
    def parse_adc_config(self, config):
        adc_dict = {}
        section = ModelConfigBase.ADC
        interval = self.check_max_interval(
            config[section].getint(ModelConfigBase.POLLING_INTERVAL, 1))
        adc_dict[ModelConfigBase.POLLING_INTERVAL] = interval

        self._configs[section] = adc_dict

    # global
    def get_default_conf(self):
        return self._configs["DEFAULT"]

    def get_log_conf(self):
        return self._configs[ModelConfigBase.LOG]

    def get_adc_conf(self):
        return self._configs[ModelConfigBase.ADC]

    def send_interval(self):
        conf = self.get_default_conf()
        return conf.get(ModelConfigBase.SEND_INTERVAL)

    # update
    def update_adc_conf(self, key, dict):
        old_conf = None
        section = ""
        if key == ModelConfigBase.ADC:
            old_conf = self.get_adc_conf()
            section = ModelConfigBase.ADC
        else:
            self._logging.error("ADC config key is invalid.")
            return None

        for key, value in dict.items():
            old_conf[key] = value

        self._configs[section] = old_conf
        return section

    def update_config(self, data):
        update_list = {}

        for key, value in data.items():
            section = None
            if "_config" not in key:
                continue

            index = key.find('_config')
            key = key[:index]

            if "ADC" in key:
                section = self.update_adc_conf(key, value)

            if section is not None:
                update_list[section] = self._configs[section]

        return update_list

#
# End of File
#
